use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Astrophysical
CXC
CloneClear
Constants
DB
DDL
Diab
Failure
Field
FieldType
Intro
Jerius
Manual
NonEmptyStr
Observatory
ScalarRef
Smithsonian
Str
Table
Types
Util
djerius
lib
nullable
